@@ -41,6 +41,8 @@
 #include <QDebug>
 #include <QFileDialog>
 #include <QFileInfo>
+#include <QMaemo5InformationBox>
+
 // gstreamer includes
 #include <gst/gst.h>
 #include <gst/gstbin.h>
@@ -61,6 +63,9 @@
 #include "pluginloaderthread.h"
 
 
+extern QTime debugSpeedTimer;
+
+
 const QDBusArgument & operator<<(QDBusArgument &arg, const Property &change)
 {
         arg.beginStructure();
@@ -85,15 +90,15 @@ MaemoBarcodeWindow::MaemoBarcodeWindow(QWidget *parent)
     pluginsLoaded = false;
 
     // first of all, disable the camera app:
-    qDebug() << "MainWindow::MainWindow(): Killing the camera app";
+    qDebug() << "MainWindow::MainWindow(): Killing the camera app, t=" << debugSpeedTimer.elapsed();
     QProcess process;
     process.execute("/usr/sbin/dsmetool -k /usr/bin/camera-ui");
 
-    qDebug() << "MainWindow::MainWindow(): Setting window title";
+    qDebug() << "MainWindow::MainWindow(): Setting window title, t=" << debugSpeedTimer.elapsed();
     setWindowTitle("Maemo Barcode Reader");
 
     // set up dbus
-    qDebug() << "MainWindow::MainWindow(): Setting up dbus";
+    qDebug() << "MainWindow::MainWindow(): Setting up dbus, t=" << debugSpeedTimer.elapsed();
     qDBusRegisterMetaType< Property >();
     qDBusRegisterMetaType< QList<Property> >();
     qDebug() << "MainWindow::MainWindow(): Setting up lensCoverPropertyModified";
@@ -113,7 +118,7 @@ MaemoBarcodeWindow::MaemoBarcodeWindow(QWidget *parent)
             this, SLOT(cameraButtonPropertyModified(int,QList<Property>)));
 
     // init the state of the lens cover
-    qDebug() << "MainWindow::MainWindow(): Initing state of lens cover";
+    qDebug() << "MainWindow::MainWindow(): Initing state of lens cover, t=" << debugSpeedTimer.elapsed();
     initLensCoverState();
 
     // make sure the events are not called to pause the pipeline unecessary in the beginning
@@ -123,44 +128,44 @@ MaemoBarcodeWindow::MaemoBarcodeWindow(QWidget *parent)
     // makes sure we don't try to do anything with a pointer to nowhere (using null-checks)
     videoWidget = NULL;
 
-    qDebug() << "MainWindow::MainWindow(): Creating settings";
+    qDebug() << "MainWindow::MainWindow(): Creating settings, t=" << debugSpeedTimer.elapsed();
     settings = new QSettings("mbarcode", "mbarcode"); // load settings
     if(settings->value("settings_version", 1).toInt() < SETTINGS_VERSION) {
         settings->clear();
         settings->setValue("settings_version", SETTINGS_VERSION);
     }
 
-    qDebug() << "MainWindow::MainWindow(): Initializing decoder";
+    qDebug() << "MainWindow::MainWindow(): Initializing decoder, t=" << debugSpeedTimer.elapsed();
     decoder = new BarcodeDetector(); // create a decoder
 
-    qDebug() << "MainWindow::MainWindow(): Initializing results window";
-    resultsWindow = new ResultsWindow(this); // create a resultswindow
+    //qDebug() << "MainWindow::MainWindow(): Initializing results window, t=" << debugSpeedTimer.elapsed();
+    //resultsWindow = new ResultsWindow(this); // create a resultswindow
 
     // set up the videoWidget
-    qDebug() << "MainWindow::MainWindow(): Initializing video widget";
+    qDebug() << "MainWindow::MainWindow(): Initializing video widget, t=" << debugSpeedTimer.elapsed();
     videoWidget = new VideoWidget(this, decoder);
-    qDebug() << "MainWindow::MainWindow(): Setting video widget attributes";
+    qDebug() << "MainWindow::MainWindow(): Setting video widget attributes, t=" << debugSpeedTimer.elapsed();
     videoWidget->setAttribute(Qt::WA_NativeWindow, true);
     videoWidget->setContinuousFocusTime((int)(settings->value("autofocus_delay",2.0).toFloat()*1000)); // pull from the settings
-    qDebug() << "MainWindow::MainWindow(): Finished setting video widget attributes";
+    qDebug() << "MainWindow::MainWindow(): Finished setting video widget attributes, t=" << debugSpeedTimer.elapsed();
     //    videoWidget->setMinimumSize(300, 200);
 
     ////videoWidget->setMinimumSize(640,400);
 
     // connect the signal from the barcode decoder
-    qDebug() << "MainWindow::MainWindow(): Connecting signals";
+    qDebug() << "MainWindow::MainWindow(): Connecting signals, t=" << debugSpeedTimer.elapsed();
     connect(decoder, SIGNAL(imageAnalysed(QString, QString, QImage*)),
             this, SLOT(imageAnalysed(QString, QString, QImage*)));
     // connect signals to disable/enable the refocus button
     connect(videoWidget, SIGNAL(focusingValueChanged(bool)), this, SLOT(setFocusing(bool)));
     connect(videoWidget, SIGNAL(doSaveBuffer()), this, SLOT(saveBufferImage()));
 
-    qDebug() << "MainWindow::MainWindow(): Initializing buttons";
+    qDebug() << "MainWindow::MainWindow(): Initializing buttons, t=" << debugSpeedTimer.elapsed();
     setAttribute(Qt::WA_Maemo5StackedWindow);
     setAttribute(Qt::WA_Maemo5AutoOrientation, true); // currently not working since libqt4-maemo5 is broken. Will work in PR1.2
 
     // if the lens cover is open, start scanning
-    qDebug() << "MainWindow::MainWindow(): Checking lens cover state";
+    qDebug() << "MainWindow::MainWindow(): Checking lens cover state, t=" << debugSpeedTimer.elapsed();
     if (isLensCoverOpen()) {
         videoWidget->start(true); // start and keep running
     } else {
@@ -168,12 +173,16 @@ MaemoBarcodeWindow::MaemoBarcodeWindow(QWidget *parent)
         // wait until the lens cover is opened.
     }
 
-    qDebug() << "MainWindow::MainWindow(): Setting central widget";
+    qDebug() << "MainWindow::MainWindow(): Setting central widget, t=" << debugSpeedTimer.elapsed();
     this->setCentralWidget(videoWidget);
-    qDebug() << "MainWindow::MainWindow(): Finished";
+    //qDebug() << "MainWindow::MainWindow(): starting pluginloader thread, t=" << debugSpeedTimer.elapsed();
 
-    pluginloaderthread = new PluginLoaderThread(this);
-    pluginloaderthread->run();
+    //pluginloaderthread = new PluginLoaderThread(this);
+    //pluginloaderthread->start(QThread::LowPriority);
+
+    qDebug() << "MainWindow::MainWindow(): Finished, t=" << debugSpeedTimer.elapsed();
+
+    //QTimer::singleShot(2000, this, SLOT(loadPlugins())); // set 2s as a start delay time
 }
 
 MaemoBarcodeWindow::~MaemoBarcodeWindow() {
@@ -293,12 +302,86 @@ void MaemoBarcodeWindow::ensurePluginsLoaded()
 {
     if (!pluginsLoaded)
     {
+        QMaemo5InformationBox::information (this, "Loading plugins (first time only)...", 10000);
+        loadPlugins();
+        //pluginloaderthread->wait();
+        QMaemo5InformationBox::information (this, "Loaded plugins", 1);
+
+        pluginsLoaded = true; // set this now so that if the timer elapses it will return and we can call loadPlugins() here
         //loadPlugins();
-        pluginloaderthread->wait();
-        //pluginsLoaded = true;
+
     }
 }
 
+void MaemoBarcodeWindow::loadPlugins() {
+    // TODO: Create a system which lets the user enable/disable plugins. These should not be loaded.
+
+    if (pluginsLoaded) // if we were too late and ensurePluginsLoaded() was called...
+        return;
+
+    qDebug() << "MainWindow::MainWindow(): Initializing results window, t=" << debugSpeedTimer.elapsed();
+    resultsWindow = new ResultsWindow(this); // create a resultswindow
+
+    qApp->processEvents();
+
+    qDebug() << "Loading plugins..., t=" << debugSpeedTimer.elapsed();
+
+    //    pluginsDir = QDir(qApp->applicationDirPath());.
+    QDir pluginsDir = QDir("/usr/share/mbarcode/");
+    QDir pluginsDirHome = QDir(QDir::homePath() + "/.config/mbarcode/plugins");
+
+    // some path stuff derived from Nokia's plugin tutorials - not really necessary in a Maemo app, but we'll just leave them here
+#if defined(Q_OS_WIN)
+    if (pluginsDir.dirName().toLower() == "debug" || pluginsDir.dirName().toLower() == "release")
+        pluginsDir.cdUp();
+#elif defined(Q_OS_MAC)
+    if (pluginsDir.dirName() == "MacOS") {
+        pluginsDir.cdUp();
+        pluginsDir.cdUp();
+        pluginsDir.cdUp();
+    }
+#endif
+    pluginsDir.cd("plugins");
+    QStringList allPluginFiles;
+
+    foreach (QString fileName, pluginsDir.entryList(QDir::Files)) { // go through all files in the plugin dir
+        allPluginFiles << pluginsDir.absoluteFilePath(fileName);
+    }
+
+    qApp->processEvents();
+
+    foreach (QString fileName, pluginsDirHome.entryList(QDir::Files)) { // go through all files in the plugin dir in home
+        allPluginFiles << pluginsDirHome.absoluteFilePath(fileName);
+    }
+
+    qApp->processEvents();
+
+    foreach (QString fileName, allPluginFiles) { // go through all files in the plugin dir
+        qDebug(qPrintable("Loading " + fileName));
+        QPluginLoader loader(fileName); // create a loader for the shared library
+        QObject *plugin = loader.instance();  // load it as a QObject
+        if (plugin) { // if it is a plugin (this fails if virtual functions are not implemented correct)
+            qDebug(qPrintable("Loaded successfully: " + fileName));
+            pluginFileNames += fileName; // list all our plugin file names
+            PluginInterface* interface = qobject_cast<PluginInterface*>(plugin); // cast it to our kind of interface
+            qDebug(qPrintable("Interface name: " + interface->getName()));
+            pluginInterfaces.append(interface); // add this interface to our list of interfaces
+            interface->initInterface(this); // initialize the plugin
+            foreach(PluginAction* pluginAction, interface->getPluginActions()) { // get the list of sink plugins in the interface
+                qDebug(qPrintable("Loading sink plugin " + pluginAction->getName()));
+                pluginActions.append(pluginAction); // add them to our list
+            }
+            qApp->processEvents();
+        } else {
+            qWarning(qPrintable("Could not load: " + fileName + " due to error: " + loader.errorString()));
+        }
+    }
+    qDebug() << "Done loading plugins, t=" << debugSpeedTimer.elapsed();
+    resultsWindow->setPluginActions(pluginActions); // let the resultwindow have all our plugins
+    pluginsLoaded = true;
+}
+
+
 //void MaemoBarcodeWindow::loadPlugins() {
 //    // TODO: Create a system which lets the user enable/disable plugins. These should not be loaded.
 //
@@ -391,7 +474,7 @@ void MaemoBarcodeWindow::imageAnalysed(QString barcodeType, QString barcodeData,
 }
 
 void MaemoBarcodeWindow::showResultsWindow(QString barcodeData) {
-    //ensurePluginsLoaded(); // not needed as we call it earlier
+    ensurePluginsLoaded(); // not needed as we call it earlier, put it back anyway just in case
     this->videoWidget->stopTimedScan();
     videoWidget->stop(false);
     resultsWindow->show();
