@@ -38,6 +38,7 @@
 #include <QtGui>
 #include <QSettings>
 #include <QHash>
+#include <QTimer>
 
 //#include <dbus/dbus.h>
 #include <QtDBus>
@@ -116,10 +117,10 @@ protected:
 private:
 
 
-
+    void loadPlugins();
     RecentDialog *recentDialog;
 
-    void loadPlugins();
+
     void initLensCoverState();
 
     bool focusing;
@@ -129,7 +130,7 @@ private:
 
 
 
-    PluginLoaderThread *pluginloaderthread;
+    //PluginLoaderThread *pluginloaderthread;
 
 private slots:
 //    void on_actionTest_portrait_mode_triggered();
@@ -142,6 +143,7 @@ private slots:
     void saveBufferImage();
 
     void keyPressEvent(QKeyEvent*);
+    //
 };
 
 #endif // MAINWINDOW_H
