@@ -41,6 +41,8 @@
 #include <QTimer>
 #include <QItemSelectionModel>
 
+#define PLUGIN_COUNT_LIMIT    6
+
 ResultsWindow::ResultsWindow(QWidget *parent) :
         QMainWindow(parent),
         ui(new Ui::ResultsWindow)
@@ -88,16 +90,29 @@ void ResultsWindow::processBarcode(QString barcode) {
         ui->lblResults->setText(tr("Results for '%1'").arg(barcode).simplified());
     }
 
+    PluginUpdateCounter = 0;
+
     checkPlugins();
 
     // save the barcode text
     this->barcode_text = QString(barcode);
     checkTimer->start(1000); // check for updated plugins every x milliseconds
 }
+
+
+
+// This API is not ideal as it results in our continuing to check plugins when they may have already decided they don't want to be used
+// it would probably be better IMO to have two functions isReady() and isInterested()
+// Once a plugin has decided it's not interested in the payload we can exclude it from the checking list (save time/power)
+// and eventually we'll know whether all the plugins have become ready or not
+// in the meantime, I'll limit the plugin checkin with a timer.
+
 void ResultsWindow::checkPlugins() {
 //    qDebug() << "Checking plugins";
     bool addedNewPlugins=false;
 
+    PluginUpdateCounter++;
+
     foreach(PluginAction* plugin, plugins) {
 //        qDebug() << "Checking action plugin " << plugin->getName() << "is ready" << plugin->isReady();
         if(plugin->isReady() && !addedPlugins.contains(plugin)) { // check if it's ready and that it has not yet been added
@@ -115,6 +130,11 @@ void ResultsWindow::checkPlugins() {
         ui->tblResults->resizeRowsToContents();
         ui->tblResults->setFixedHeight(ui->tblResults->verticalHeader()->length() + 60); // fix to make all contents of scrollwidget scroll together
     }
+
+    if(PluginUpdateCounter > PLUGIN_COUNT_LIMIT)
+        checkTimer->stop();
+
+
 }
 
 void ResultsWindow::on_pushButton_clicked()
