@@ -115,14 +115,21 @@ VideoWidget::VideoWidget(MaemoBarcodeWindow *parent, BarcodeDetector *decoder) :
     qDebug() << "VideoWidget::VideoWidget(): Setting up frame counts";
     // timer, should be set last, just in case
     frames = 0;
-    frametime.start();
+    //frametime.start(); // doesn't seem to do anything
     timerOpenGL = new QTimer(this);
     QObject::connect(timerOpenGL, SIGNAL(timeout()), this, SLOT(updateGL()));
-    if(parent->isLensCoverOpen())
+
+
+    // will need to change this to not have the timerOpenGL running at all if the lenscover is closed at startup
+    if(parent->isLensCoverOpen()){
         timerOpenGL->setInterval(PAINTGL_INTERVAL_HIGH); //update the window every x milliseconds
-    else
+        //timerOpenGL->start();
+    }else{
         timerOpenGL->setInterval(PAINTGL_INTERVAL_LOW); //update the window every x milliseconds
-    timerOpenGL->start();
+        //this->paintGL();
+    }
+    timerOpenGL->start(); // if I place this inside the if construct above we never get any video if we startup with the lens cover closed!?!
+
     qDebug() << "VideoWidget::VideoWidget(): Finished";
 }
 
@@ -335,10 +342,15 @@ void VideoWidget::paintGL() {
         painter.endNativePainting();
 
         // Draw a pretty line to pretend we are a barcode scanner
-        QPen redPen;
-        redPen.setColor(QColor(150,10,10,150));
-        redPen.setWidth(3);
-        painter.setPen(redPen);
+        QPen aPen;
+        if(decoder->scanning_status){ // we are currently scanning
+            aPen.setColor(QColor(10,150,10,150));
+            aPen.setWidth(3);
+        }else{ // we're not scanning atm
+            aPen.setColor(QColor(150,10,10,150));
+            aPen.setWidth(3);
+        }
+        painter.setPen(aPen);
         painter.drawLine(QPoint(0,height() / 2), QPoint(width(), height() / 2));
         // Draw some pretty rectangles to guide our user
         QBrush blackBrush;
@@ -378,6 +390,7 @@ void VideoWidget::paintGL() {
     glDisable(GL_DEPTH_TEST);
     glDisable(GL_CULL_FACE);
 
+    // we should alter this somehow so that when we paint the last frame before stopping the timer, we show e.g. 0fps
     QString framesPerSecond;
     framesPerSecond.setNum(frames /(time.elapsed() / 1000.0), 'f', 2);
 
@@ -421,6 +434,9 @@ gboolean VideoWidget::busCallback_callback (GstBus * bus, GstMessage * message)
     return videoWidget_callback->busCallback(bus,message);
 }
 // End listing callback functions
+
+
+
 void VideoWidget::start(bool evenIfNotYetStarted) {
     qDebug() << "VideoWidget::start(): Called with startEvenIfNotYetStarted =" << evenIfNotYetStarted << "and lensCoverOpen =" << parent->isLensCoverOpen();
     if((evenIfNotYetStarted || keepRunning) && parent->isLensCoverOpen()) {
@@ -435,8 +451,12 @@ void VideoWidget::start(bool evenIfNotYetStarted) {
             refocusTimer->start();
         }        
         keepRunning = true;
-        timerOpenGL->setInterval(PAINTGL_INTERVAL_HIGH);
-    }
+        timerOpenGL->start(PAINTGL_INTERVAL_HIGH);
+        //timerOpenGL->setInterval(PAINTGL_INTERVAL_HIGH);
+    }else
+        if(!timerOpenGL->isActive())
+            timerOpenGL->start(PAINTGL_INTERVAL_HIGH);
+
     // see if the settings wants us to have continuous scanning enabled
     if(parent->settings->value("continous_scanning",CONTINUOUS_SCANNING_DEFAULT).toBool()){
         setContinuousScanning(true);
@@ -458,7 +478,11 @@ void VideoWidget::stop(bool stopCompletely) {
     if(stopCompletely) {
         keepRunning = false;
     }
-    timerOpenGL->setInterval(PAINTGL_INTERVAL_LOW);
+    //timerOpenGL->setInterval(PAINTGL_INTERVAL_LOW);
+    timerOpenGL->stop();
+
+    // call the paint method one last time to update the screen
+    this->paintGL();
 
 }
 bool VideoWidget::isPipelineReady() {
@@ -932,3 +956,6 @@ void VideoWidget::setContinuousScanning(bool on)
 {
     decoder->scanning_status=on;
 }
+
+
+
